from datetime import datetime
import time
from anypubsub.interfaces import PubSub, Subscriber
from pymongo import CursorType


class MongoSubscriber(Subscriber):
    def __init__(self, collection, channels):
        import pymongo
        pymongo_version = int(pymongo.__version__.split('.')[0])
        self.cursor = collection.find({'channel': {'$in': channels}, 'when': {'$gte': datetime.utcnow()}},
                                      cursor_type = CursorType.TAILABLE_AWAIT)

    def __iter__(self):
        return self

    def next(self):
        while self.cursor.alive:
            try:
                message = next(self.cursor)
                if message['type'] == 'message':
                    return message['message']
            except StopIteration:  # pragma: no cover
                time.sleep(1)

    __next__ = next  # PY3


class MongoPubSub(PubSub):
    def __init__(self, host=None, port=None, maxPoolSize=100,
                 client=None, database='anypubsub', collection='anyps_messages',
                 collection_size=10 * 2 ** 20):
        self.api = MongoPubSub._api()
        if client is None:
            client = self.api.MongoClient(host=host, port=port, maxPoolSize=maxPoolSize)
        db = client[database]
        try:
            db.create_collection(collection, size=collection_size, capped=True)
        except self.api.errors.CollectionInvalid:
            pass
        self.collection = db[collection]

    @staticmethod
    def _api():
        return __import__('pymongo')

    def publish(self, channel, message):
        self.collection.insert({'type': 'message', 'channel': channel, 'message': message, 'when': datetime.utcnow()})

    def subscribe(self, *channels):
        return MongoSubscriber(self.collection, channels)


backend = MongoPubSub




